#ifndef BUBBLEWINDOW_INCLUDED
#define BUBBLEWINDOW_INCLUDED

/* responsible for the window */
#include <GLES2/gl2.h>
#include "algebra.h"
#include "envCube.h"
#include "shape.h"

#include "nvtexfont2.h"

typedef enum {false = 0, true} bool;

typedef struct {
    int time_frames;
    int frame_count;
    double startTime;
    double currentTime;
    float duration;
    int autopoke;
    int autopoke_framenum;
    float elapsed;
    int fps_flag;
    int swapInterval;
    int mouse_flag;
    bool one_frame;
    unsigned int cube_texture;
    Shape *bubble;
    int window_width, window_height;
    int mouse_x, mouse_y;
    float screen_x, screen_y;
    bool mouse_down;
    float aspect;
    float near_height;
    float near_distance;
    float far_distance;
    float eye_distance;
    enum Mode {POINT_MODE, WIRE_MODE, CUBE_MODE} mode;
    EnvCube* env_cube;
    float heading;
    float pitch;
    float hv, pv;
    Quat orient;

    NVTexfont2RasterFont *nvtxf;
} BubbleWindow;

void NewBubbleWindow(BubbleWindow *b, int autopoke, float duration, int fpsFlag);
void BubbleWindow_reshapeViewport(BubbleWindow *b, GLsizei w, GLsizei h);
void BubbleWindow_mouse(BubbleWindow *b, GLsizei x, GLsizei y);
void BubbleWindow_leftMouse(BubbleWindow *b, bool click);
void BubbleWindow_init(BubbleWindow *b, GLsizei w, GLsizei h);
void BubbleWindow_callback(BubbleWindow *b, int key);
void BubbleWindow_tick(BubbleWindow *b);
void DestroyBubbleWindow(BubbleWindow *b);

#endif //BUBBLEWINDOW_INCLUDED
