/*
 * nvgl2demo_common.h
 *
 * Copyright 2007-2009, NVIDIA Corporation.
 * All rights Reserved.
 *
 * THE INFORMATION CONTAINED HEREIN IS PROPRIETARY AND CONFIDENTIAL TO
 * NVIDIA, CORPORATION.  USE, REPRODUCTION OR DISCLOSURE TO ANY THIRD PARTY
 * IS SUBJECT TO WRITTEN PRE-APPROVAL BY NVIDIA, CORPORATION.
 */

#ifndef __NVGL2DEMO_COMMON_H
#define __NVGL2DEMO_COMMON_H

// The header files all the demos use
#include <EGL/egl.h>
#include <EGL/eglext.h>
#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

// Opaque platform specific information
typedef struct NvGl2DemoPlatformState NvGl2DemoPlatformState;
    
// Window System and EGL objects
typedef struct NvGl2DemoGfxStateRec {
    NativeDisplayType nativeDisplay;
    NativeWindowType  nativeWindow;
    EGLDisplay        display;
    EGLSurface        surface;
    EGLConfig         config;
    EGLContext        context;
    EGLint            width;
    EGLint            height;
    EGLint            buffering;
    EGLenum           api;          // EGL_OPENGL_ES_API, EGL_OPENVG_API, ...
    NvGl2DemoPlatformState* platformspecific;
} NvGl2DemoGfxState;
extern NvGl2DemoGfxState gfxState;

// Window system/egl initialization/shutdown functions
int  NvGl2DemoInitialize(int* argc, char** argv, const char *appName,
                         EGLint eglApiBit);
void NvGl2DemoShutdown(void);

// helper function returning usage string
char * NvGl2DemoGetUsageString(void);

// Pixmap allocation
void *NvGl2DemoCreatePixmap(unsigned int width, unsigned int height,
                            unsigned int depth);

// Presetting and querying Platform Specific state
int   NvGl2DemoPresetPlatformState(const char *state, void *value);
void *NvGl2DemoGetPlatformState(const char *state);

// Event handling
typedef void (*NvGl2DemoCloseCB)(void);
typedef void (*NvGl2DemoResizeCB)(int w, int h);
typedef void (*NvGl2DemoKeyCB)(char key, int state);
typedef void (*NvGl2DemoPointerCB)(int x, int y);
typedef void (*NvGl2DemoButtonCB)(int button, int state);
typedef void (*NvGl2DemoExtEventHandler)( const KDEvent* inEvent );
void NvGl2DemoSetCloseCB(NvGl2DemoCloseCB cb);
void NvGl2DemoSetResizeCB(NvGl2DemoResizeCB cb);
void NvGl2DemoSetKeyCB(NvGl2DemoKeyCB cb);
void NvGl2DemoSetPointerCB(NvGl2DemoPointerCB cb);
void NvGl2DemoSetButtonCB(NvGl2DemoButtonCB cb);
void NvGl2DemoSetExtEventHandler(NvGl2DemoExtEventHandler cb);
void NvGl2DemoCheckEvents(void);

// Shader loading functions
GLuint NvGl2DemoLoadShaderBinaries(
    const char* vertBin, GLuint vertBinSize,
    const char* fragBin, GLuint fragBinSize,
    GLboolean debugging);
GLuint NvGl2DemoLoadShaderSources(
    const char* vertFile,                       
    const char* fragFile,
    GLboolean debugging);


// Math stuff
#define eps 1e-4

int eq(float a, float b);


// Matrix functions
void matrix_identity(float m[16]);
int matrix_equals(float a[16], float b[16]);
void matrix_transpose(float m[16]);
void matrix_multiply(float m0[16], float m1[16]);
void matrix_multiply_4x4_3x3(float m0[16], float m1[9]);
void matrix_multiply_3x3(float m0[9], float m1[9]);
void matrix_frustum(float m[16],
                    float l, float r, float b, float t, float n, float f);
void matrix_ortho(float m[16],
                  float l, float r, float b, float t, float n, float f);
void matrix_translate(float m[16], float x, float y, float z);
void matrix_rotate_create3x3(float m[9],
                             float theta, float x, float y, float z);
void matrix_rotate(float m[16], float theta, float x, float y, float z);
void matrix_rotate_3x3(float m[9], float theta, float x, float y, float z);

float matrix_determinant(float m[16]);
void matrix_inverse(float m[16]);
void matrix_copy(float dest[16], float src[16]);

void matrix_print(float a[16]);

void matrix_vector_multiply(float m[16],  float v[4]);

/* some #defs for useful timeouts */
#define NV_WETIME_ONEMS         ((KDust)1*1000*1000)
#define NV_WETIME_DEFAULT       NV_WETIME_ONEMS
#define NV_WETIME_60FPS         (NV_WETIME_ONEMS * 15) /* assume 1ms for frame... */
#define NV_WETIME_30FPS         (NV_WETIME_ONEMS * 30) /* assume 1ms per frame.. */
#define NV_WETIME_1FPS          (NV_WETIME_ONEMS * 1000)
#define NV_WETIME_NEVER         0
#define NV_WETIME_ALWAYS        ((KDust)-1)

#ifdef __cplusplus
}
#endif

#endif // __NVGL2DEMO_COMMON_H
