#ifndef SHAPE_INCLUDED
#define SHAPE_INCLUDED

#include "algebra.h"

#define MAX_DEPTH 64

typedef struct {
    float p[3];  // point
    float n[3];  // normal
    float v[3];  // velocity
    float h[3];  // home
    float a[3];  // neighborhood velocity for averaging
    int connectedness; // indicates how many triangles this vertex is
                       // connected to, as well as how many edges
} Vertex;

typedef struct {
    short v0id;
    short v1id;
    float l; // length
} Edge;

typedef struct {
    int numVerts;  // numTris == numVerts-2
    Vertex **vertices;
    unsigned short *indices;
} Tristrip;

typedef struct {
    int numTristrips;
    Tristrip *tristrips;
    int numEdges;
    Edge *edges;
    int numVerts;
    Vertex *vertices;
    float final_drag;
    float initial_drag;
    float drag;
} Shape;

Shape *buildSphere(int depth);
void Bubble_destroy(Shape *b);
void Bubble_draw(Shape *b, unsigned int cube_texture);
void Bubble_calcNormals(Shape *b);
void Bubble_pick(Shape *b, const float3 e, const float3 n);
void Bubble_calcVelocity(Shape *b);
void Bubble_filterVelocity(Shape *b);
void Bubble_drawVertices(Shape *b);
void Bubble_drawEdges(Shape *b);

#endif //SHAPE_INCLUDED
